@@ -18,11 +18,9 @@ package org.apache.catalina.filters;
 
 import java.io.IOException;
 import java.io.Serializable;
-import java.security.SecureRandom;
 import java.util.HashSet;
 import java.util.LinkedHashMap;
 import java.util.Map;
-import java.util.Random;
 import java.util.Set;
 
 import javax.servlet.FilterChain;
@@ -48,43 +46,14 @@ import org.apache.juli.logging.LogFactory;
  * returned to the client
  * </ul>
  */
-public class CsrfPreventionFilter extends FilterBase {
+public class CsrfPreventionFilter extends CsrfPreventionFilterBase {
+    private final Log log = LogFactory.getLog(CsrfPreventionFilter.class);
 
-    private static final Log log =
-        LogFactory.getLog(CsrfPreventionFilter.class);
-
-    private String randomClass = SecureRandom.class.getName();
-
-    private Random randomSource;
-
-    private int denyStatus = HttpServletResponse.SC_FORBIDDEN;
-
-    private final Set<String> entryPoints = new HashSet<String>();
+    private final Set<String> entryPoints = new HashSet<>();
 
     private int nonceCacheSize = 5;
 
-    @Override
-    protected Log getLogger() {
-        return log;
-    }
-
-    /**
-     * Return response status code that is used to reject denied request.
-     */
-    public int getDenyStatus() {
-        return denyStatus;
-    }
-
-    /**
-     * Set response status code that is used to reject denied request. If none
-     * set, the default value of 403 will be used.
-     *
-     * @param denyStatus
-     *            HTTP status code
-     */
-    public void setDenyStatus(int denyStatus) {
-        this.denyStatus = denyStatus;
-    }
+    private String nonceRequestParameterName = Constants.CSRF_NONCE_REQUEST_PARAM;
 
     /**
      * Entry points are URLs that will not be tested for the presence of a valid
@@ -117,13 +86,13 @@ public class CsrfPreventionFilter extends FilterBase {
     }
 
     /**
-     * Specify the class to use to generate the nonces. Must be in instance of
-     * {@link Random}.
+     * Sets the request parameter name to use for CSRF nonces.
      *
-     * @param randomClass   The name of the class to use
+     * @param parameterName The request parameter name to use
+     *        for CSRF nonces.
      */
-    public void setRandomClass(String randomClass) {
-        this.randomClass = randomClass;
+    public void setNonceRequestParameterName(String parameterName) {
+        this.nonceRequestParameterName = parameterName;
     }
 
     @Override
@@ -131,22 +100,10 @@ public class CsrfPreventionFilter extends FilterBase {
         // Set the parameters
         super.init(filterConfig);
 
-        try {
-            Class<?> clazz = Class.forName(randomClass);
-            randomSource = (Random) clazz.newInstance();
-        } catch (ClassNotFoundException e) {
-            ServletException se = new ServletException(sm.getString(
-                    "csrfPrevention.invalidRandomClass", randomClass), e);
-            throw se;
-        } catch (InstantiationException e) {
-            ServletException se = new ServletException(sm.getString(
-                    "csrfPrevention.invalidRandomClass", randomClass), e);
-            throw se;
-        } catch (IllegalAccessException e) {
-            ServletException se = new ServletException(sm.getString(
-                    "csrfPrevention.invalidRandomClass", randomClass), e);
-            throw se;
-        }
+        // Put the expected request parameter name into the application scope
+        filterConfig.getServletContext().setAttribute(
+                Constants.CSRF_NONCE_REQUEST_PARAM_NAME_KEY,
+                nonceRequestParameterName);
     }
 
     @Override
@@ -161,104 +118,179 @@ public class CsrfPreventionFilter extends FilterBase {
             HttpServletRequest req = (HttpServletRequest) request;
             HttpServletResponse res = (HttpServletResponse) response;
 
-            boolean skipNonceCheck = false;
-
-            if (Constants.METHOD_GET.equals(req.getMethod())) {
-                String path = req.getServletPath();
-                if (req.getPathInfo() != null) {
-                    path = path + req.getPathInfo();
-                }
-
-                if (entryPoints.contains(path)) {
-                    skipNonceCheck = true;
-                }
-            }
-
             HttpSession session = req.getSession(false);
 
-            @SuppressWarnings("unchecked")
-            LruCache<String> nonceCache = (session == null) ? null
-                    : (LruCache<String>) session.getAttribute(
-                            Constants.CSRF_NONCE_SESSION_ATTR_NAME);
+            boolean skipNonceCheck = skipNonceCheck(req);
+            NonceCache<String> nonceCache = null;
 
             if (!skipNonceCheck) {
-                String previousNonce =
-                    req.getParameter(Constants.CSRF_NONCE_REQUEST_PARAM);
+                String previousNonce = req.getParameter(nonceRequestParameterName);
 
-                if (nonceCache == null || previousNonce == null ||
-                        !nonceCache.contains(previousNonce)) {
-                    res.sendError(denyStatus);
+                if (previousNonce == null) {
+                    if (log.isDebugEnabled()) {
+                        log.debug("Rejecting request for " + getRequestedPath(req)
+                                  + ", session "
+                                  + (null == session ? "(none)" : session.getId())
+                                  + " with no CSRF nonce found in request");
+                    }
+
+                    res.sendError(getDenyStatus());
                     return;
                 }
-            }
 
-            if (nonceCache == null) {
-                nonceCache = new LruCache<String>(nonceCacheSize);
-                if (session == null) {
-                    session = req.getSession(true);
+                nonceCache = getNonceCache(req, session);
+                if (nonceCache == null) {
+                    if (log.isDebugEnabled()) {
+                        log.debug("Rejecting request for " + getRequestedPath(req)
+                                  + ", session "
+                                  + (null == session ? "(none)" : session.getId())
+                                  + " due to empty / missing nonce cache");
+                    }
+
+                    res.sendError(getDenyStatus());
+                    return;
+                } else if (!nonceCache.contains(previousNonce)) {
+                    if (log.isDebugEnabled()) {
+                        log.debug("Rejecting request for " + getRequestedPath(req)
+                                  + ", session "
+                                  + (null == session ? "(none)" : session.getId())
+                                  + " due to invalid nonce " + previousNonce);
+                    }
+
+                    res.sendError(getDenyStatus());
+                    return;
+                }
+                if (log.isTraceEnabled()) {
+                    log.trace("Allowing request to " + getRequestedPath(req)
+                               + " with valid CSRF nonce " + previousNonce);
                 }
-                session.setAttribute(
-                        Constants.CSRF_NONCE_SESSION_ATTR_NAME, nonceCache);
             }
 
-            String newNonce = generateNonce();
+            if (!skipNonceGeneration(req)) {
+                if (skipNonceCheck) {
+                    // Didn't look up nonce cache earlier so look it up now.
+                    nonceCache = getNonceCache(req, session);
+                }
+                if (nonceCache == null) {
+                    if (log.isDebugEnabled()) {
+                        log.debug("Creating new CSRF nonce cache with size=" + nonceCacheSize + " for session " + (null == session ? "(will create)" : session.getId()));
+                    }
 
-            nonceCache.add(newNonce);
+                    if (session == null) {
+                        if (log.isDebugEnabled()) {
+                             log.debug("Creating new session to store CSRF nonce cache");
+                        }
 
-            wResponse = new CsrfResponseWrapper(res, newNonce);
-        } else {
-            wResponse = response;
+                        session = req.getSession(true);
+                    }
+
+                    nonceCache = createNonceCache(req, session);
+                }
+
+                String newNonce = generateNonce(req);
+
+                nonceCache.add(newNonce);
+
+                // Take this request's nonce and put it into the request
+                // attributes so pages can make direct use of it, rather than
+                // requiring the use of response.encodeURL.
+                request.setAttribute(Constants.CSRF_NONCE_REQUEST_ATTR_NAME, newNonce);
+
+                wResponse = new CsrfResponseWrapper(res, nonceRequestParameterName, newNonce);
+            }
         }
 
-        chain.doFilter(request, wResponse);
+        chain.doFilter(request, wResponse == null ? response : wResponse);
     }
 
 
-    @Override
-    protected boolean isConfigProblemFatal() {
+    protected boolean skipNonceCheck(HttpServletRequest request) {
+        if (!Constants.METHOD_GET.equals(request.getMethod())) {
+            return false;
+        }
+
+        String requestedPath = getRequestedPath(request);
+
+        if (!entryPoints.contains(requestedPath)) {
+            return false;
+        }
+
+        if (log.isTraceEnabled()) {
+            log.trace("Skipping CSRF nonce-check for GET request to entry point " + requestedPath);
+        }
+
         return true;
     }
 
 
     /**
-     * Generate a once time token (nonce) for authenticating subsequent
-     * requests. This will also add the token to the session. The nonce
-     * generation is a simplified version of ManagerBase.generateSessionId().
+     * Determines whether a nonce should be created. This method is provided
+     * primarily for the benefit of sub-classes that wish to customise this
+     * behaviour.
      *
+     * @param request   The request that triggered the need to potentially
+     *                      create the nonce.
+     *
+     * @return {@code true} if a nonce should be created, otherwise
+     *              {@code false}
      */
-    protected String generateNonce() {
-        byte random[] = new byte[16];
+    protected boolean skipNonceGeneration(HttpServletRequest request) {
+        return false;
+    }
 
-        // Render the result as a String of hexadecimal digits
-        StringBuilder buffer = new StringBuilder();
 
-        randomSource.nextBytes(random);
+    /**
+     * Create a new {@link NonceCache} and store in the {@link HttpSession}.
+     * This method is provided primarily for the benefit of sub-classes that
+     * wish to customise this behaviour.
+     *
+     * @param request   The request that triggered the need to create the nonce
+     *                      cache. Unused by the default implementation.
+     * @param session   The session associated with the request.
+     *
+     * @return A newly created {@link NonceCache}
+     */
+    protected NonceCache<String> createNonceCache(HttpServletRequest request, HttpSession session) {
 
-        for (int j = 0; j < random.length; j++) {
-            byte b1 = (byte) ((random[j] & 0xf0) >> 4);
-            byte b2 = (byte) (random[j] & 0x0f);
-            if (b1 < 10) {
-                buffer.append((char) ('0' + b1));
-            } else {
-                buffer.append((char) ('A' + (b1 - 10)));
-            }
-            if (b2 < 10) {
-                buffer.append((char) ('0' + b2));
-            } else {
-                buffer.append((char) ('A' + (b2 - 10)));
-            }
+        NonceCache<String> nonceCache = new LruCache<>(nonceCacheSize);
+
+        session.setAttribute(Constants.CSRF_NONCE_SESSION_ATTR_NAME, nonceCache);
+
+        return nonceCache;
+    }
+
+
+    /**
+     * Obtain the {@link NonceCache} associated with the request and/or session.
+     * This method is provided primarily for the benefit of sub-classes that
+     * wish to customise this behaviour.
+     *
+     * @param request   The request that triggered the need to obtain the nonce
+     *                      cache. Unused by the default implementation.
+     * @param session   The session associated with the request.
+     *
+     * @return The {@link NonceCache} currently associated with the request
+     *         and/or session
+     */
+    protected NonceCache<String> getNonceCache(HttpServletRequest request, HttpSession session) {
+        if (session == null) {
+            return null;
         }
-
-        return buffer.toString();
+        @SuppressWarnings("unchecked")
+        NonceCache<String> nonceCache =
+                (NonceCache<String>) session.getAttribute(Constants.CSRF_NONCE_SESSION_ATTR_NAME);
+        return nonceCache;
     }
 
     protected static class CsrfResponseWrapper
             extends HttpServletResponseWrapper {
 
+        private final String nonceRequestParameterName;
         private final String nonce;
 
-        public CsrfResponseWrapper(HttpServletResponse response, String nonce) {
+        public CsrfResponseWrapper(HttpServletResponse response, String nonceRequestParameterName, String nonce) {
             super(response);
+            this.nonceRequestParameterName = nonceRequestParameterName;
             this.nonce = nonce;
         }
 
@@ -284,16 +316,15 @@ public class CsrfPreventionFilter extends FilterBase {
             return addNonce(super.encodeURL(url));
         }
 
-        /**
+        /*
          * Return the specified URL with the nonce added to the query string.
          *
          * @param url URL to be modified
-         * @param nonce The nonce to add
          */
         private String addNonce(String url) {
 
             if ((url == null) || (nonce == null)) {
-                return (url);
+                return url;
             }
 
             String path = url;
@@ -316,15 +347,23 @@ public class CsrfPreventionFilter extends FilterBase {
             } else {
                 sb.append('?');
             }
-            sb.append(Constants.CSRF_NONCE_REQUEST_PARAM);
+            sb.append(nonceRequestParameterName);
             sb.append('=');
             sb.append(nonce);
             sb.append(anchor);
-            return (sb.toString());
+            return sb.toString();
         }
     }
 
-    protected static class LruCache<T> implements Serializable {
+
+    protected static interface NonceCache<T> extends Serializable {
+        void add(T nonce);
+
+        boolean contains(T nonce);
+    }
+
+
+    protected static class LruCache<T> implements NonceCache<T> {
 
         private static final long serialVersionUID = 1L;
 
@@ -345,12 +384,14 @@ public class CsrfPreventionFilter extends FilterBase {
             };
         }
 
+        @Override
         public void add(T key) {
             synchronized (cache) {
                 cache.put(key, null);
             }
         }
 
+        @Override
         public boolean contains(T key) {
             synchronized (cache) {
                 return cache.containsKey(key);
